@@ -4,9 +4,9 @@ import sqlite3
 from conan.api.output import ConanOutput
 from conan.internal.cache.db.packages_table import PackagesDBTable
 from conan.internal.cache.db.recipes_table import RecipesDBTable
-from conans.model.package_ref import PkgReference
-from conans.model.recipe_ref import RecipeReference
-from conans.model.version import Version
+from conan.api.model import PkgReference
+from conan.api.model import RecipeReference
+from conan.internal.model.version import Version
 
 
 class CacheDatabase:
@@ -99,3 +99,9 @@ class CacheDatabase:
     def get_package_references(self, ref: RecipeReference, only_latest_prev=True):
         return [d["pref"]
                 for d in self._packages.get_package_references(ref, only_latest_prev)]
+
+    def path_to_ref(self, path):
+        ref = self._recipes.path_to_ref(path)
+        if ref is not None:
+            return ref
+        return self._packages.path_to_ref(path)
